using System;
using System.Collections.Generic;
using System.Linq;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.GamerServices;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Media;
using Microsoft.Xna.Framework.Net;
using Microsoft.Xna.Framework.Storage;

namespace LjudIXNA
{
    /// <summary>
    /// This is the main type for your game
    /// </summary>
    public class Game1 : Microsoft.Xna.Framework.Game
    {
        GraphicsDeviceManager graphics;
        SpriteBatch spriteBatch;
        Texture2D line;
        VisualizationData visData = new VisualizationData();

        KeyboardState prev_ks = new KeyboardState();
        AudioEngine audioEngine;
        WaveBank waveBank;
        SoundBank soundBank;

        Cue throttleSound;

        public Game1()
        {
            graphics = new GraphicsDeviceManager(this);
            Content.RootDirectory = "Content";
        }

        /// <summary>
        /// Allows the game to perform any initialization it needs to before starting to run.
        /// This is where it can query for any required services and load any non-graphic
        /// related content.  Calling base.Initialize will enumerate through any components
        /// and initialize them as well.
        /// </summary>
        protected override void Initialize()
        {
            // TODO: Add your initialization logic here

            base.Initialize();
        }

        /// <summary>
        /// LoadContent will be called once per game and is the place to load
        /// all of your content.
        /// </summary>
        protected override void LoadContent()
        {
            // Create a new SpriteBatch, which can be used to draw textures.
            spriteBatch = new SpriteBatch(GraphicsDevice);

            line = Content.Load<Texture2D>("line");

            MediaPlayer.Volume = 0.1f;
            MediaPlayer.IsRepeating = true;
            MediaPlayer.Play(Content.Load<Song>("music"));
            MediaPlayer.IsVisualizationEnabled = true;

            audioEngine = new AudioEngine("Content\\sounds.xgs");
            waveBank = new WaveBank(audioEngine, "Content\\Wave Bank.xwb");
            soundBank = new SoundBank(audioEngine, "Content\\Sound Bank.xsb");
            throttleSound = soundBank.GetCue("throttle");
        }

        /// <summary>
        /// UnloadContent will be called once per game and is the place to unload
        /// all content.
        /// </summary>
        protected override void UnloadContent()
        {
            // TODO: Unload any non ContentManager content here
        }

        /// <summary>
        /// Allows the game to run logic such as updating the world,
        /// checking for collisions, gathering input, and playing audio.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        protected override void Update(GameTime gameTime)
        {
            // Allows the game to exit
            if (GamePad.GetState(PlayerIndex.One).Buttons.Back == ButtonState.Pressed)
                this.Exit();

            KeyboardState ks = Keyboard.GetState();

            if (ks.IsKeyDown(Keys.F1) && prev_ks.IsKeyUp(Keys.F1))
            {
                soundBank.PlayCue("explosion");
            }

            if (ks.IsKeyDown(Keys.F2))
            {
                if (!throttleSound.IsPlaying)
                    throttleSound.Play();
                else if (throttleSound.IsPaused)
                    throttleSound.Resume();
            }
            else
            {
                if (throttleSound.IsPlaying)
                    throttleSound.Pause();
            }

            prev_ks = ks;

            base.Update(gameTime);
        }

        /// <summary>
        /// This is called when the game should draw itself.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        protected override void Draw(GameTime gameTime)
        {
            GraphicsDevice.Clear(Color.CornflowerBlue);
            spriteBatch.Begin();

            if (MediaPlayer.State == MediaState.Playing)
                MediaPlayer.GetVisualizationData(visData);

            int height = 100;

            for (int x = 0; x < visData.Frequencies.Count; x++)
            {
                // Frekvensspektrum
                spriteBatch.Draw(line, 
                    new Vector2(x, (float)((-visData.Frequencies[x]) * height)) + new Vector2(100, 200), Color.White);
                // Vgform
                spriteBatch.Draw(line,
                    new Vector2(x, (float)(visData.Samples[x] * height)) + new Vector2(100, 400), Color.White);
            }
            spriteBatch.End();
            base.Draw(gameTime);
        }
    }
}
